import numpy as np
import argparse
import os

parser = argparse.ArgumentParser()
parser.add_argument('--fea', type=str, default='features/', help='features_path')
parser.add_argument('--lst', type=str, default='./list_casp13', help='pdb input id')
args = parser.parse_args()


def text_save(filename, data):
    """
    :param filename: The path where the file is written
    :param data: List of written data
    :return:
    """
    file = open(filename, 'w')
    for i in range(len(data)):
        lst = data[i]
        s = str(data[i]).replace('[', '').replace(']', '')  # remove []
        s = s.replace("'", '').replace(',', '') + '\n'      # Remove single quotes, commas, and append a newline at the end of each line
        file.write(s)
    file.close()
   
    
def txt_read(filename):
    """
    :param filename:
    :return: a list containing the elements of each line
    """
    with open(filename, 'r') as fid:
        lines = fid.read().splitlines()
        while '' in lines:
            lines.remove('')
    return lines
  
    
def create_dir_if_not_exists(directory):
    if not os.path.exists(directory):
        os.makedirs(directory)
        
        
def maes(id, pphi, phi, ppsi, psi, ss, save_path):
    """
    Calculate the MAE of a chain of alpha and belta, coil.
    :param id:          str, protein id
    :param pphi:        list, predicted phi
    :param phi:         list, true phi
    :param ppsi:        list, predicted psi
    :param psi:         list, true psi
    :param ss:          list, secondary structure
    :param save_path:   str, save path for id.mae
    :return:
    """
    num_alpha_belta = 0
    lst_coil = []
    nb_outputs = ['PHI', 'PSI']
    maes = [[] for i in range(len(nb_outputs))]
    pred = [pphi, ppsi]
    true = [phi, psi]
    # print(len(ss[0]))
    for i in range(0, len(ss)):
        for j in range(len(nb_outputs)):
            # if ss[i] not in lst_coil:
            if ss[i] not in lst_coil and float(true[j][i]) != 360.0:
                num_alpha_belta += 1
                d = np.absolute(float(pred[j][i]) - float(true[j][i]))
                maes[j].append(min(360 - d, d))
    MAE = [np.average(_maes) for _maes in maes]

    f = open(save_path + id + '.mae', 'w')
    print("protein: {}, sequence length: {}, Q8 for alpha and belta: {}".format(id, len(ss), num_alpha_belta // 2),
          file=f)
    print("protein: {}, sequence length: {}, [Phi, Psi]: {}".format(id, len(ss), MAE))
    print(MAE, file=f)
    sum_mae = 0
    for i in range(0, len(MAE)):
        sum_mae += MAE[i]
    if sum_mae < 20 + 30:
        print("Better prediction: {}".format(id), file=f)
    return MAE


def maes_alpha_belta(id, pphi, phi, ppsi, psi, ss, save_path):
    """
    Calculate the MAE of a chain of alpha and belta, don't calculate the coil.
    :param id:          str, protein id
    :param pphi:        list, predicted phi
    :param phi:         list, true phi
    :param ppsi:        list, predicted psi
    :param psi:         list, true psi
    :param ss:          list, secondary structure
    :param save_path:   str, save path for id.mae
    :return:
    """
    num_alpha_belta = 0
    lst_coil = ['T', 'S', 'C']
    nb_outputs = ['PHI', 'PSI']
    maes = [[] for i in range(len(nb_outputs))]
    pred = [pphi, ppsi]
    true = [phi, psi]
    for i in range(0, len(ss)):
        for j in range(len(nb_outputs)):
            if ss[i] not in lst_coil:
                num_alpha_belta += 1
                d = np.absolute(float(pred[j][i]) - float(true[j][i]))
                maes[j].append(min(360 - d, d))
    MAE = [np.average(_maes) for _maes in maes]

    f = open(save_path + id + '.mae', 'w')
    print("protein: {}, sequence length: {}, Q8 for alpha and belta: {}".format(id, len(ss), num_alpha_belta // 2),
          file=f)
    print("protein: {}, sequence length: {}, [Phi, Psi]: {}".format(id, len(ss), MAE))
    print(MAE, file=f)
    sum_mae = 0
    for i in range(0, len(MAE)):
        sum_mae += MAE[i]
    if sum_mae < 14 + 16:
        print("Better prediction: {}".format(id), file=f)


def evaluate(pphsi_path, model, dataset, mode, nb_outputs, cuda=None, device_id=None):
    """2 MAE"""
    if mode == TRAIN_MODE: nb_examples = dataset.train.size
    if mode == DEV_MODE: nb_examples = dataset.dev.size
    if mode == TEST_MODE: nb_examples = dataset.test.size

    maes = [[] for i in range(nb_outputs)]
    maes_xy = [[] for i in range(nb_outputs)]
    for _ in range(nb_examples):
        ids, lengths, inputs, _, _, angles = dataset.next_batch(1, mode)
        if cuda: inputs = inputs.cuda(device_id)
        outputs = model(inputs, lengths).squeeze().cpu().data.numpy()
        outputs = outputs * 2.0 - 1.0

        mae = [[] for i in range(nb_outputs)]
        pre_angles = [[] for i in range(nb_outputs)]  # predicted angles
        for i in range(lengths[0]):
            for j in range(nb_outputs):
                predicted_sin, predicted_cos = outputs[i, j * 2], outputs[i, j * 2 + 1] + 1e-8
                predicted_angle = np.rad2deg(np.arctan2(predicted_sin, predicted_cos))
                pre_angles[j].append('%.4f' % predicted_angle)
                correct_angle = angles[0][i][j]
                if correct_angle != 360:
                    d = np.absolute(predicted_angle - correct_angle)
                    maes[j].append(min(360 - d, d))
                    mae[j].append(min(360 - d, d))
        mae_ = [np.average(x) for x in mae]
        [maes_xy[k].append(mae_[k]) for k in range(2)]
        # save predicted phi to file
        create_dir_if_not_exists(pphsi_path)
        text_save(pphsi_path + ids[0] + '.pphi', pre_angles[0])
        text_save(pphsi_path + ids[0] + '.ppsi', pre_angles[1])
    MAE = [np.average(_maes) for _maes in maes]
    Ave_mae = [np.average(x) for x in maes_xy]
    return MAE, Ave_mae
    

def main(lst):
    k = 0
    sum = len(lst)
    create_dir_if_not_exists(args.fea + 'maes/')
    ave_maes = [[], []]
    lst_worse = []
    for id in lst:
        id = id[:5]
        k += 1
        #print('protein: {}, {}/{}'.format(id, k, sum))
        pphi = txt_read(args.fea + 'pphsi/' + id + '.pphi')
        phi = txt_read(args.fea + 'phsi/' + id + '.phi')
        ppsi = txt_read(args.fea + 'pphsi/' + id + '.ppsi')
        psi = txt_read(args.fea + 'phsi/' + id + '.psi')
        ss = txt_read(args.fea + 'ss/' + id + '.ss')
        #print(ss)
        # maes_alpha_belta(id, pphi, phi, ppsi, psi, ss, args.save_mae)
        MAE = maes(id, pphi, phi, ppsi, psi, ss, args.fea + 'maes/')
        
        [ave_maes[j].append(MAE[j]) for j in range(2)]
        if float(MAE[0] + MAE[1]) > 60:
          lst_worse.append(id)
    mae = [np.average(_maes) for _maes in ave_maes]
    print("{} proteins mae: {}".format(k, mae))
    text_save(args.fea + 'lst_badpred', lst_worse)

if __name__ == '__main__':
    lst = txt_read(args.lst)[0:]
    main(lst)