import os
import csv
import math
import pandas as pd
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
import numpy as np
import argparse
import datetime

from reader import Dataset, TRAIN_MODE, DEV_MODE, TEST_MODE

SUPPORTED_INPUT_TYPES = ['ONEHOT', 'PHYS', 'PSSM', 'PC',  'RE', 'DC', 'RAMA'] 
SUPPORTED_OUTPUT_TYPES = ['PHI', 'PSI']

parser = argparse.ArgumentParser(description='Testing Model')
parser.add_argument('--input_types', nargs='+', default=['PSSM', 'ONEHOT', 'PHYS', 'RE', 'DC', 'PC', 'RAMA'], help='Input types')
parser.add_argument('--output_types', nargs='+', default=['PHI', 'PSI'], help='Output types')
parser.add_argument('--test_model', type=str, default='model/esiden1', help='Model type')
parser.add_argument('--readfile', type=str, default='input/data_casp11/', help='Input file path')

parser.add_argument('--cuda', type=str, default='true', help='Cuda usage')
parser.add_argument('--device_id', type=int, default=0, help='GPU Device ID number')
parser.add_argument('--results_path', type=str, default='newresults/data_all_shuffle/', help='results output path')
parser.add_argument('--pphsi_path', type=str, default='features/casp14/pphsi/', help='predicted pphsi path')


def create_dir_if_not_exists(directory):
    if not os.path.exists(directory):
        os.makedirs(directory)


def text_save(filename, data):
    """
    :param filename: The path where the file is written
    :param data: List of written data
    :return:
    """
    file = open(filename, 'w')
    for i in range(len(data)):
        lst = data[i]
        s = str(data[i]).replace('[', '').replace(']', '')
        s = s.replace("'", '').replace(',', '') + '\n'
        file.write(s)
    file.close()
    
    
def evaluate(pphsi_path, model, dataset, mode, nb_outputs, cuda=None, device_id=None):
    """calculate maes for dev,test"""
    if mode == TRAIN_MODE: nb_examples = dataset.train.size
    if mode == DEV_MODE: nb_examples = dataset.dev.size
    if mode == TEST_MODE: nb_examples = dataset.test.size

    maes = [[] for i in range(nb_outputs)]
    for _ in range(nb_examples):
        ids, lengths, inputs, _, _, angles = dataset.next_batch(1, mode)
        if cuda: inputs = inputs.cuda(device_id)
        outputs = model(inputs, lengths).squeeze().cpu().data.numpy()
        outputs = outputs * 2.0 - 1.0
        pre_angles = [[] for i in range(nb_outputs)]  # predicted angles
        for i in range(lengths[0]):
            for j in range(nb_outputs):
                predicted_sin, predicted_cos = outputs[i, j * 2], outputs[i, j * 2 + 1] + 1e-8
                predicted_angle = np.rad2deg(np.arctan2(predicted_sin, predicted_cos))
                pre_angles[j].append('%.4f' % predicted_angle)
                correct_angle = angles[0][i][j]
                if correct_angle != 360:
                    d = np.absolute(predicted_angle - correct_angle)
                    maes[j].append(min(360 - d, d))
        # save predicted phi to file, uncommon here
        #create_dir_if_not_exists(pphsi_path)
        #text_save(pphsi_path + ids[0] + '.pphi', pre_angles[0])
        #text_save(pphsi_path + ids[0] + '.ppsi', pre_angles[1])
    MAE = [np.average(_maes) for _maes in maes]
    return MAE

    
# Main Function
def main():
    # Arguments Parsing
    global args
    args = parser.parse_args()
    cuda = args.cuda
    if cuda == 'true' and torch.cuda.is_available():
        cuda = True
    else:
        cuda = False
    device_id = args.device_id
    input_types = args.input_types
    output_types = args.output_types

    # Checks
    for input_type in input_types:
        assert(input_type in SUPPORTED_INPUT_TYPES)
    for output_type in output_types:
        assert(output_type in SUPPORTED_OUTPUT_TYPES)

    # Load dataset
    print('Loading dataset ...')
    dataset = Dataset(input_types, output_types, args.readfile)
    print('Loading model ...')
    model = torch.load(args.test_model)
    if cuda: model.cuda(device_id)
    print('Calculating MAE ...')
    test_maes, mae = evaluate(args.pphsi_path, model, dataset, TEST_MODE, len(output_types), cuda, device_id)
    print("[INFO] Test MAEs: ", test_maes, mae)


if __name__ == "__main__":
    main()
